/**
 * Copyright 2010 Trustees of the University of Pennsylvania Licensed under the
 * Educational Community License, Version 2.0 (the "License"); you may
 * not use this file except in compliance with the License. You may
 * obtain a copy of the License at
 *
 * http://www.osedu.org/licenses/ECL-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS"
 * BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */

package metridoc.dsl

import org.apache.camel.builder.RouteBuilder
import metridoc.utils.Assert
import org.apache.camel.Exchange
import org.slf4j.Logger
import org.slf4j.LoggerFactory
import org.apache.camel.model.RouteDefinition

/**
 *
 * @author Tommy Barker
 */
@Mixin(PropertyMissing.class)
class GroovyRouteBuilder extends RouteBuilder {

    def Closure route
    def JobRunner jobRunner
    def Logger log = LoggerFactory.getLogger(GroovyRouteBuilder.class)
    def Boolean usePolling

    @Override
    RouteDefinition from(String uri) {
        def newUri = uri

        if (usePolling(uri)) {
            newUri = "poll:${newUri}"
        }

        super.from newUri
    }

    def void configure() {
        Assert.notNull jobRunner, "the jobRunner cannot be null"

        onCompletion().onFailureOnly().process {Exchange exchange ->
            def exception = exchange.exception
            if (exception) {
                jobRunner.lastException = exception
            } else {
                exception = exchange.getProperty(Exchange.EXCEPTION_CAUGHT)
                if (exception) {
                    jobRunner.lastException = exception
                } else {
                    jobRunner.lastException = new RuntimeException("An unknown exception occurred, if the log is not " +
                            "sufficient to determine what happened consider setting logging level to DEBUG")
                    throw jobRunner.lastException
                }
            }
        }



        route.delegate = this
        route.setDirective(Closure.DELEGATE_ONLY)
        route()
    }

    private boolean usePolling(String uri) {

        def m = uri =~ /usePolling=(true|false)/
        def boolean result = false
        def boolean usePollingInUri

        //uri based
        if (m.find()) {
            usePollingInUri = true
            return Boolean.valueOf(m[0][1])
        }

        //global
        if (usePolling != null) {
            return usePolling
        }

        //defaults
        if (uri.startsWith("seda") || uri.startsWith("direct")) {
            return false
        } else {
            return true
        }
    }
}
