/**
 * Copyright 2010 Trustees of the University of Pennsylvania Licensed under the
 * Educational Community License, Version 2.0 (the "License"); you may
 * not use this file except in compliance with the License. You may
 * obtain a copy of the License at
 *
 * http://www.osedu.org/licenses/ECL-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS"
 * BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */
package metridoc.dsl

import metridoc.extension.ExtensionRunner
import org.slf4j.LoggerFactory
import ch.qos.logback.classic.Logger
import ch.qos.logback.classic.Level

/**
 * Created by IntelliJ IDEA.
 * User: tbarker
 * Date: 4/3/11
 * Time: 4:22 PM
 *
 *
 *
 */
class JobBuilder {

    def JobManager manager
    def log = LoggerFactory.getLogger(JobBuilder.class)

    private JobBuilder() {}

    def JobBuilder load(String... jobFiles) {
        getJobManager().parseJobs jobFiles
        return this
    }

    def JobBuilder load(Class... jobClasses) {
        getJobManager().parseJobs jobClasses
        return this
    }

    def JobManager getJobManager() {
        if(manager == null) {
            manager = ExtensionRunner.extensionRunner.jobsRunner
        }

        return manager
    }

    def void run(LinkedHashMap args=null, String... jobs) {
        configureBuilder this, args
        getJobManager().jobRunner.runJobs jobs
        checkForExceptions()
    }

    private void checkForExceptions() {
        def lastException = getJobManager().jobRunner.lastException

        if(lastException) {
            log.error "one or more exceptions occurred running jobs, the last occurring exception will be wrapped " +
                    "and thrown by the builder"
            throw new JobBuilderException("LAST EXCEPTION CAUGHT", getJobManager().jobRunner.lastException)
        }
    }

    def JobBuilder useTracing(boolean useTracing) {
        getJobManager().getJobRunner().camelContext.setTracing useTracing
    }

    private void setLoggingLevel(Level logLevel) {
        Logger logger = (Logger) LoggerFactory.getLogger(Logger.ROOT_LOGGER_NAME)
        logger.setLevel(logLevel)
    }

    def static void run(LinkedHashMap args = null, String jobName, Closure closure) {

        def builder = new JobBuilder()
        configureBuilder builder, args

        def jobManager = builder.getJobManager()
        jobManager.loadJobs closure
        jobManager.runJobs jobName
        jobManager.stop()
        builder.checkForExceptions()
    }

    private static void configureBuilder(JobBuilder builder, args) {
        if(args) {
            if(args.logLevel) {
                builder.setLoggingLevel((Level) args.logLevel)
            }

            def config = new JobBuilderConfig(args)
            config.configure builder
        }
    }

}
