/**
 * Copyright 2010 Trustees of the University of Pennsylvania Licensed under the
 * Educational Community License, Version 2.0 (the "License"); you may
 * not use this file except in compliance with the License. You may
 * obtain a copy of the License at
 *
 * http://www.osedu.org/licenses/ECL-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS"
 * BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */
package metridoc.dsl

import ch.qos.logback.classic.Level
import org.springframework.context.ApplicationContext
import org.apache.camel.spi.Registry
import org.springframework.context.annotation.AnnotationConfigApplicationContext
import org.springframework.context.support.ClassPathXmlApplicationContext
import org.apache.camel.spring.spi.ApplicationContextRegistry

import metridoc.registry.MetridocSimpleRegistry
import metridoc.registry.RegistryWrapper
import ch.qos.logback.classic.Logger
import org.slf4j.LoggerFactory

/**
 * Created by IntelliJ IDEA.
 * User: tbarker
 * Date: 4/16/11
 * Time: 10:57 PM
 *
 * represents the config for a {@link JobBuilder} instance.  Amongst all the spring configs only one should be set
 *
 */
class JobBuilderConfig {
    def boolean shutDownCamel = true
    def boolean useTracing = false
    def Level logLevel = Level.INFO
    def String springXml
    def Class springConfig
    def ApplicationContext springContext
    /**
     * not supported yet, planning on using the groovy bean builder
     */
    def Class springGroovyClass
    /**
     * not supported yet, planning on using the groovy bean builder
     */
    def String springGroovyFile
    def Registry registry

    def void configure(JobBuilder builder) {
        validateState()
        def springContext = getContext()
        def springRegistry
        if(springContext) {
            springRegistry = new ApplicationContextRegistry(springContext)
        }

        def services = builder.getJobManager().services
        def simpleRegistry = new MetridocSimpleRegistry(services)
        def wrapper = new RegistryWrapper()

        if(springRegistry) {
            wrapper.registries.add springRegistry
        }

        wrapper.registries.add simpleRegistry
        builder.getJobManager().registry = wrapper
    }

    private ApplicationContext getContext() {
        if(springConfig) return new AnnotationConfigApplicationContext(springConfig)
        if(springContext) return springContext
        if(springXml) return new ClassPathXmlApplicationContext(springXml)

        return null
    }

    private void validateState() {
        int springCount = 0

        if(springConfig) springCount++
        if(springContext) springCount++
        if(springXml) springCount++
        if(springGroovyClass) springCount++
        if(springGroovyFile) springCount++

        if(springCount > 1) {
            def message = "more than one spring configuration was passed, only one can be passed"
            throw new IllegalStateException(message)
        }
    }
}
