/**
 * Copyright 2010 Trustees of the University of Pennsylvania Licensed under the
 * Educational Community License, Version 2.0 (the "License"); you may
 * not use this file except in compliance with the License. You may
 * obtain a copy of the License at
 *
 * http://www.osedu.org/licenses/ECL-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS"
 * BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */

package metridoc.extension

import metridoc.utils.IOUtils;
import metridoc.dsl.JobManager;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory

/**
 *
 * @author Tommy Barker
 */
class ExtensionRunner {
    private static final String DEFAULT_EXTENSION = "MetridocExtension.groovy";
    private static final Logger LOG = LoggerFactory.getLogger(ExtensionRunner.class);
    private ClassLoader classLoader;
    private Binding binding = new Binding();
    private static boolean extensionsHaveRun = false;
    def JobManager jobsRunner = new JobManager();

    /**
     * extension manager should be a singleton and only implemented once
     */
    private ExtensionRunner() {}

    public static ExtensionRunner getExtensionRunner() {
        def extensionRunner = new ExtensionRunner()

        if (!extensionsHaveRun) {
            extensionRunner.runExtensions(DEFAULT_EXTENSION)
        }

        return extensionRunner;
    }

    public Binding getBinding() {
        return binding;
    }

    public void setBinding(Binding binding) {
        this.binding = binding;
    }


    public ClassLoader getClassLoader() {
        if (classLoader == null) {
            classLoader = ExtensionRunner.class.getClassLoader();
        }

        return classLoader;
    }

    public void setClassLoader(ClassLoader classLoader) {
        this.classLoader = classLoader;
    }

    public void runExtensions(String... extensions) throws Exception {
        runExtensions(binding, extensions);
    }

    private synchronized void runExtensions(Binding binding, String... extensions) throws Exception {
        for (String extension : extensions) {
            LOG.debug("searching for extension {}", extension);
            List<InputStream> resources = IOUtils.getResources(extension, getClassLoader());
            if (resources.isEmpty()) {
                LOG.warn("could not find any extensions for {}", extension);
            }

            for (InputStream inputStream : resources) {
                GroovyShell shell = new GroovyShell(binding);
                shell.evaluate(new InputStreamReader(inputStream));
            }
        }

        Map variables = binding.getVariables();
        for (Object key : variables.keySet()) {
            jobsRunner.getServices().put((String) key, variables.get(key));
        }
    }
}

