/**
 * Copyright 2010 Trustees of the University of Pennsylvania Licensed under the
 * Educational Community License, Version 2.0 (the "License"); you may
 * not use this file except in compliance with the License. You may
 * obtain a copy of the License at
 *
 * http://www.osedu.org/licenses/ECL-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS"
 * BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */
package metridoc.project

import groovy.text.SimpleTemplateEngine;

import java.io.IOException;
import java.io.File;

import org.apache.commons.io.FileUtils;

/**
 * Created by IntelliJ IDEA.
 * User: tbarker
 * Date: 4/11/11
 * Time: 10:33 AM
 *
 * This creates a project structure for a new metridoc instance.  The project structure should look like:
 *
 * <directory>/
 *      pom.xml
 *      src/
 *          main/
 *              groovy/
 *                  <jobName>.groovy
 *          test/
 *              groovy/
 *
 */
class ProjectBuilder {

	private static REL_DIRS_TO_CREATE = [
		"src/main/groovy",
		"src/test/groovy"
	];

	def String groupId
	def String artifactId
	def String directory
	def String jobName

	def void build() {
		def String fullPath;
		for(v in REL_DIRS_TO_CREATE){
			fullPath = directory + File.separator + v;
			def File dirToCreate =  new File(fullPath);
			if(!dirToCreate.mkdirs() && !dirToCreate.exists()){
				throw new IOException("Cannot create required directory: " + fullPath);
			}
		}

		def binding = [
					groupId : groupId,
					artifactId  : artifactId,
					jobName : jobName
				]

		def loaderParent = this.class.classLoader;
		def InputStream resourceStream = loaderParent.getResourceAsStream("pom_template.xml")
		def engine = new SimpleTemplateEngine()
		def template = engine.createTemplate(new InputStreamReader(resourceStream)).make(binding)
		FileUtils.writeStringToFile new File(directory + File.separator + "pom.xml"), template.toString()
	}
}

