/**
 * Copyright 2010 Trustees of the University of Pennsylvania Licensed under the
 * Educational Community License, Version 2.0 (the "License"); you may
 * not use this file except in compliance with the License. You may
 * obtain a copy of the License at
 *
 * http://www.osedu.org/licenses/ECL-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS"
 * BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */
package metridoc.schema.loader

import org.springframework.core.io.Resource
import org.springframework.core.io.ResourceLoader
import groovy.xml.MarkupBuilder
import metridoc.utils.SchemaUtils

/**
 * Created by IntelliJ IDEA.
 * User: tbarker
 * Date: 4/28/11
 * Time: 12:43 PM
 * To change this template use File | Settings | File Templates.
 */
class GroovyClassSchemaResource implements Resource, ResourceLoader{

    String className
    ClassLoader classLoader

    ClassLoader getClassLoader() {
        if(classLoader == null) {
            classLoader = Thread.currentThread().contextClassLoader
        }

        return classLoader
    }

    boolean exists() {
        try {
            getClassLoader().loadClass(className)
            return true
        } catch (ClassNotFoundException ex) {
            return false
        }
    }

    boolean isReadable() {
        return false
    }

    boolean isOpen() {
        return false
    }

    URL getURL() {
        return null
    }

    URI getURI() {
        return null
    }

    File getFile() {
        return null
    }

    long contentLength() {
        throw new UnsupportedOperationException("not supported")
    }

    long lastModified() {
        throw new UnsupportedOperationException("not supported")
    }

    Resource createRelative(String relativePath) {
        throw new UnsupportedOperationException("not supported")
    }

    String getFilename() {
        return className
    }

    String getDescription() {
        return className
    }

    InputStream getInputStream() {
        def schema = getClassLoader().loadClass(className).newInstance()
        def writer = new StringWriter()
        def xml = new MarkupBuilder(writer)
        def xmlClosure
        def schemaUtils = new SchemaUtils()

        schema.metaClass.schema = {Closure closure ->
            xmlClosure = closure
        }
        schema.metaClass.include = {Class clazz ->
            include(file:"${clazz.name}.xml")
        }
        schema.metaClass.methodMissing = {String name, args ->
            xml.invokeMethod name, args
        }
        schema.metaClass.propertyMissing = {String name ->
            if(name == "schemaUtils") {
                return schemaUtils
            }
        }
        schemaUtils.metaClass.methodMissing = {String name, args ->
            xml.invokeMethod name, args
        }
        schema.run()

        xmlClosure.setDelegate(xml)
        xmlClosure.setDelegate(Closure.DELEGATE_ONLY)
        xml.databaseChangeLog(xmlns: 'http://www.liquibase.org/xml/ns/dbchangelog',
                        'xmlns:xsi': 'http://www.w3.org/2001/XMLSchema-instance',
                        'xsi:schemaLocation': 'http://www.liquibase.org/xml/ns/dbchangelog http://www.liquibase.org/xml/ns/dbchangelog/dbchangelog-2.0.xsd',
                    xmlClosure)

        def bytes = writer.toString().getBytes()
        return new ByteArrayInputStream(bytes)
    }

    Resource getResource(String location) {
        if(location.endsWith(".xml")) {
            location = location.substring(0, location.length() - 4)
        }
        return new GroovyClassSchemaResource(className: location)
    }
}
